@@ -6,7 +6,6 @@
 #include "log.h"
 #include "rand.h"
 #include "chunk.h"
-#include "h2.h"             /* h2_send_1xx() */
 #include "http_range.h"     /* http_range_config_allow_http10() */
 #include "fdevent.h"
 #include "fdlog.h"
@@ -15,10 +14,10 @@
 #include "stat_cache.h"
 #include "plugin.h"
 #include "plugins.h"
-#include "plugin_config.h"  /* config_plugin_value_tobool() */
+#include "plugin_config.h"
 #include "network_write.h"  /* network_write_show_handlers() */
 #include "reqpool.h"        /* request_pool_init() request_pool_free() */
-#include "response.h"       /* http_response_send_1xx_cb_set() strftime_cache_reset() */
+#include "response.h"       /* http_dispatch[] strftime_cache_reset() */
 
 #ifdef HAVE_VERSIONSTAMP_H
 # include "versionstamp.h"
@@ -31,22 +30,87 @@ static const buffer default_server_tag =
   { PACKAGE_DESC "\0server", sizeof(PACKAGE_DESC), 0 };
 
 #include <sys/types.h>
-#include <sys/stat.h>
 #include "sys-setjmp.h"
+#include "sys-stat.h"
 #include "sys-time.h"
+#include "sys-unistd.h" /* <unistd.h> */
 
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <locale.h>
+#ifdef _WIN32
+#include <mbctype.h>    /* _setmbcp() */
+#endif
 
 #include <stdio.h>
 
 #ifdef HAVE_GETOPT_H
 # include <getopt.h>
+#else
+/* basic (very limited) getopt() implementation */
+extern char *optarg;
+extern int optind, opterr, optopt;
+char *optarg = NULL;
+int optind = 1, opterr = 1, optopt = 0;
+int getopt (int argc, char * const argv[], const char *optstring);
+int getopt (int argc, char * const argv[], const char *optstring)
+{
+    static char *nextchar;
+    optarg = NULL;
+    if (optind >= argc || argc < 1)
+        return -1;
+    if (optind <= 1)
+        nextchar = argv[(optind = 1)];
+    else if (nextchar == NULL)
+        nextchar = argv[optind];
+
+    if (nextchar == argv[optind]) {
+        if (*nextchar++ != '-'
+            || nextchar[0] == '\0' /* "-" */
+            || (nextchar[0] == '-' && nextchar[1] == '\0')) { /* "--" */
+            return -1;
+        }
+        ++optind;
+    }
+
+    const char *o = optstring;
+    if (*o == '+' || *o == '-') ++o; /*(ignore; behave as if '+' is set)*/
+    if (*o == ':') ++o;              /*(ignore; behave as if ':' is set)*/
+    for (; *o; ++o) {
+        if (*o == *nextchar)
+            break;
+        if (o[1] == ':') ++o;
+        if (o[1] == ':') ++o;
+    }
+    if (!*o) {
+        /* if (opterr) fprintf(stderr, "..."); */
+        optopt = *nextchar;
+        return '?';
+    }
+
+    if (!*++nextchar)
+        nextchar = NULL;
+
+    if (o[1] == ':') {
+        if (nextchar) {
+            optarg = nextchar;
+            nextchar = NULL;
+        }
+        else if (optind < argc)
+            optarg = argv[optind++];
+        else if (o[2] != ':') {
+            /* if (opterr) fprintf(stderr, "..."); */
+              /*(fprintf if ':' not at beginning of optstring)*/
+            optopt = *o;
+            return ':';
+        }
+    }
+
+    return *o;
+}
 #endif
 
 #ifdef HAVE_VALGRIND_VALGRIND_H
@@ -100,11 +164,19 @@ static size_t malloc_top_pad;
 #define TEXT_SSL
 #endif
 
-#ifndef __sgi
-/* IRIX doesn't like the alarm based time() optimization */
-/* #define USE_ALARM */
+#ifdef _WIN32
+/* (Note: assume overwrite == 1 in this setenv() replacement) */
+/*#define setenv(name,value,overwrite)  SetEnvironmentVariable((name),(value))*/
+/*#define unsetenv(name)                SetEnvironmentVariable((name),NULL)*/
+#define setenv(name,value,overwrite)  _putenv_s((name), strdup(value))
+#define unsetenv(name)                _putenv_s((name), "")
 #endif
 
+#include "h1.h"
+static const struct http_dispatch h1_1_dispatch_table = {
+  .send_1xx          = h1_send_1xx
+};
+
 static int oneshot_fd = 0;
 static int oneshot_fdout = -1;
 static fdnode *oneshot_fdn = NULL;
@@ -120,6 +192,20 @@ static volatile sig_atomic_t handle_sig_alarm = 1;
 static volatile sig_atomic_t handle_sig_hup = 0;
 static int idle_limit = 0;
 
+__attribute_cold__
+int server_main (int argc, char ** argv);
+
+#ifdef _WIN32
+#ifndef SIGBREAK
+#define SIGBREAK 21
+#endif
+/* Ctrl-BREAK (repurposed and treated as SIGUSR1)*/
+#ifndef SIGUSR1
+#define SIGUSR1 SIGBREAK
+#endif
+#include "server_win32.c"
+#endif
+
 #if defined(HAVE_SIGACTION) && defined(SA_SIGINFO)
 static volatile siginfo_t last_sigterm_info;
 static volatile siginfo_t last_sighup_info;
@@ -186,13 +272,56 @@ static void signal_handler(int sig) {
 			graceful_shutdown = 1;
 		}
 		break;
+  #ifndef _WIN32
 	case SIGALRM: handle_sig_alarm = 1; break;
 	case SIGHUP:  handle_sig_hup = 1; break;
 	case SIGCHLD: handle_sig_child = 1; break;
+  #endif
 	}
 }
 #endif
 
+#if defined(HAVE_SIGNAL)
+#ifdef _WIN32
+static BOOL WINAPI ConsoleCtrlHandler(DWORD dwType)
+{
+    /* Note: Windows handles "signals" inconsistently, varying depending on
+     * whether or not the program is attached to a non-hidden window.
+     * CTRL_CLOSE_EVENT sent by taskkill can be received if attached to a
+     * non-hidden window, but taskkill /f must be used (and CTRL_CLOSE_EVENT
+     * is not received here) if process is not attached to a window, or if
+     * the window is hidden. (WTH MS?!)  This *does not* catch CTRL_CLOSE_EVENT:
+     *   start -FilePath .\lighttpd.exe -ArgumentList "-D -f lighttpd.conf"
+     *     -WindowStyle Hidden   # (or None)
+     * but any other -WindowStyle can catch CTRL_CLOSE_EVENT.
+     * CTRL_C_EVENT can only be sent to 0 (self process group) or self pid
+     * and is ignored by default (sending signal does not indicate failure)
+     * in numerous other cases.  Some people have resorted to standalone helper
+     * programs to attempt AttachConsole() to a target pid before sending
+     * CTRL_C_EVENT via GenerateConsoleCtrlEvent().  Another alternative is
+     * running lighttpd as a Windows service, which uses a different mechanism,
+     * also more limited than unix signals.  Other alternatives include
+     * NSSM (Non-Sucking Service Manager) or cygwin's cygrunsrv program */
+    switch(dwType) {
+      case CTRL_C_EVENT:
+        signal_handler(SIGINT);
+        break;
+      case CTRL_BREAK_EVENT:
+        /* Ctrl-BREAK (repurposed and treated as SIGUSR1)*/
+        signal_handler(SIGUSR1);
+        break;
+      case CTRL_CLOSE_EVENT:/* sent by taskkill */
+      case CTRL_LOGOFF_EVENT:
+      case CTRL_SHUTDOWN_EVENT:
+        /* non-cancellable event; program terminates soon after return */
+        signal_handler(SIGTERM);/* trigger server shutdown in main thread */
+        Sleep(2000);            /* sleep 2 secs to give threads chance to exit*/
+        return FALSE;
+    }
+    return TRUE;
+}
+#endif
+#endif
 
 static void server_main_setup_signals (void) {
   #ifdef HAVE_SIGACTION
@@ -227,25 +356,41 @@ static void server_main_setup_signals (void) {
     sigaction(SIGALRM, &act, NULL);
     sigaction(SIGUSR1, &act, NULL);
 
+   #ifdef __QNX__
+      /*
+       * In QNX SDP 7.1 SA_RESTART is not supported
+       */
+      #ifndef SA_RESTART
+         #define SA_RESTART 0
+      #endif
+   #endif /* __QNX__ */
+
     /* it should be safe to restart syscalls after SIGCHLD */
     act.sa_flags |= SA_RESTART | SA_NOCLDSTOP;
     sigaction(SIGCHLD, &act, NULL);
   #elif defined(HAVE_SIGNAL)
+   #ifndef _WIN32
     /* ignore the SIGPIPE from sendfile() */
     signal(SIGPIPE, SIG_IGN);
-    signal(SIGALRM, signal_handler);
-    signal(SIGTERM, signal_handler);
-    signal(SIGHUP,  signal_handler);
-    signal(SIGCHLD, signal_handler);
+   #endif
     signal(SIGINT,  signal_handler);
+    signal(SIGTERM, signal_handler);
+   #ifndef _WIN32
+    signal(SIGHUP,  signal_handler);
+    signal(SIGALRM, signal_handler);
     signal(SIGUSR1, signal_handler);
+    signal(SIGCHLD, signal_handler);
+   #else
+    /* Ctrl-BREAK (repurposed and treated as SIGUSR1)*/
+    signal(SIGUSR1, signal_handler);
+    SetConsoleCtrlHandler((PHANDLER_ROUTINE)ConsoleCtrlHandler,TRUE);
+   #endif
    #ifndef _MSC_VER
     signal(SIGBUS,  sys_setjmp_sigbus);
    #endif
   #endif
 }
 
-
 #ifdef HAVE_FORK
 static int daemonize(void) {
 	int pipefd[2];
@@ -303,10 +448,14 @@ static int clockid_mono_coarse = 0;
 static unix_time64_t
 server_monotonic_secs (void)
 {
+  #ifdef _MSC_VER
+    return (unix_time64_t)(GetTickCount64() / 1000);
+  #else
     unix_timespec64_t ts;
     return (0 == log_clock_gettime(clockid_mono_coarse, &ts))
       ? ts.tv_sec
       : log_monotonic_secs;
+  #endif
 }
 
 static unix_time64_t
@@ -317,15 +466,21 @@ server_epoch_secs (server * const srv, unix_time64_t mono_ts_delta)
     const unix_time64_t new_ts_adj = new_ts - mono_ts_delta;
     /* attempt to detect large clock jump */
     if (new_ts_adj < cur_ts || new_ts_adj - cur_ts > 300) { /*(5 mins)*/
-        log_error(srv->errh, __FILE__, __LINE__,
+        log_warn(srv->errh, __FILE__, __LINE__,
           "warning: clock jumped %lld secs",
           (long long)((int64_t)new_ts_adj - (int64_t)cur_ts));
+
+        /* graceful restart not available if chroot'ed */
+        if (srv->srvconf.changeroot)
+            return new_ts;
+
         int delta =                             /*(30 mins default)*/
           config_feature_int(srv, "server.clock-jump-restart", 1800);
         if (delta && (new_ts_adj > cur_ts
                       ? new_ts_adj-cur_ts
-                      : cur_ts-new_ts_adj) > delta) {
+                      : cur_ts-new_ts_adj) > (unix_time64_t)delta) {
             log_error(srv->errh, __FILE__, __LINE__,
+              "clock jumped; "
               "attempting graceful restart in < ~5 seconds, else hard restart");
             srv->graceful_expire_ts = log_monotonic_secs + 5;
             raise(SIGUSR1);
@@ -383,6 +538,7 @@ static server *server_init(void) {
 	srv->default_server_tag = &default_server_tag;
 
 	log_con_jqueue = (connection *)(uintptr_t)&log_con_jqueue;/*(sentinel)*/
+	memset(http_dispatch, 0, sizeof(http_dispatch));
 
 	return srv;
 }
@@ -393,16 +549,19 @@ static void server_free(server *srv) {
 	if (oneshot_fd > 0) {
 		if (oneshot_fdn) {
 			fdevent_fdnode_event_del(srv->ev, oneshot_fdn);
-			fdevent_unregister(srv->ev, oneshot_fd);
+			fdevent_unregister(srv->ev, oneshot_fdn);
 			oneshot_fdn = NULL;
 		}
-		close(oneshot_fd);
+		if (oneshot_fdout >= 0)
+			fdio_close_pipe(oneshot_fd);
+		else
+			fdio_close_socket(oneshot_fd);
 	}
 	if (oneshot_fdout >= 0) {
-		close(oneshot_fdout);
+		fdio_close_pipe(oneshot_fdout);
 	}
 	if (srv->stdin_fd >= 0) {
-		close(srv->stdin_fd);
+		fdio_close_socket(srv->stdin_fd);
 	}
 
 	buffer_free(srv->tmp_buf);
@@ -591,7 +750,7 @@ static int server_oneshot_init_pipe(server *srv, int fdin, int fdout) {
 
     /* note: existing routines assume socket, not pipe
      * connections.c:connection_read_cq()
-     *   uses recv() ifdef __WIN32
+     *   uses recv() ifdef _WIN32
      *   passes S_IFSOCK to fdevent_ioctl_fionread()
      *   (The routine could be copied and modified, if required)
      * This is unlikely to work if TLS is used over pipe since the SSL_CTX
@@ -637,7 +796,7 @@ static int server_oneshot_init(server *srv, int fd) {
 	}
 
 	/*(must set flags; fd did not pass through fdevent accept() logic)*/
-	if (-1 == fdevent_fcntl_set_nb_cloexec(fd)) {
+	if (-1 == fdevent_socket_set_nb_cloexec(fd)) {
 		log_perror(srv->errh, __FILE__, __LINE__, "fcntl()");
 		return 0;
 	}
@@ -764,11 +923,19 @@ static void show_features (void) {
 #else
       "\t- PAM support\n"
 #endif
+#if !defined(HAVE_SYS_INOTIFY_H) && !defined(HAVE_KQUEUE)
 #ifdef HAVE_FAM_H
       "\t+ FAM support\n"
 #else
       "\t- FAM support\n"
 #endif
+#endif
+#ifdef HAVE_SYS_INOTIFY_H
+      "\t+ inotify support\n"
+#endif
+#ifdef HAVE_KQUEUE
+      "\t+ kqueue support\n"
+#endif
 #ifdef HAVE_LUA_H
       "\t+ LUA support\n"
 #else
@@ -888,7 +1055,7 @@ static void server_sockets_close (server *srv) {
         server_socket *srv_socket = srv->srv_sockets.ptr[i];
         if (-1 == srv_socket->fd) continue;
         if (2 != srv->sockets_disabled) network_unregister_sock(srv,srv_socket);
-        close(srv_socket->fd);
+        fdio_close_socket(srv_socket->fd);
         srv_socket->fd = -1;
         /* network_close() will cleanup after us */
     }
@@ -898,6 +1065,7 @@ static void server_sockets_close (server *srv) {
 __attribute_cold__
 static void server_graceful_signal_prev_generation (void)
 {
+  #ifdef HAVE_FORK
     const char * const prev_gen = getenv("LIGHTTPD_PREV_GEN");
     if (NULL == prev_gen) return;
     pid_t pid = (pid_t)strtol(prev_gen, NULL, 10);
@@ -905,6 +1073,7 @@ static void server_graceful_signal_prev_generation (void)
     if (pid <= 0) return; /*(should not happen)*/
     if (pid == fdevent_waitpid(pid,NULL,1)) return; /*(pid exited; unexpected)*/
     kill(pid, SIGINT); /* signal previous generation for graceful shutdown */
+  #endif
 }
 
 __attribute_cold__
@@ -961,14 +1130,16 @@ static int server_graceful_state_bg (server *srv) {
          * future: fdevent kqueue and stat_cache kqueue would need to be closed,
          *         re-opened, and active fds re-registered.  Not current done.
          *         Need to create some routines like fdevent_reinit_after_fork*/
-        log_error(srv->errh, __FILE__, __LINE__,
+        log_warn(srv->errh, __FILE__, __LINE__,
           "server.graceful-restart-bg ignored on OpenBSD and NetBSD "
           "due to limitation in kqueue inheritance and lacking rfork");
         return 0;
     }
    #endif
-  #else
+  #elif defined(HAVE_FORK)
     pid_t pid = fork();
+  #else
+    pid_t pid = -1;
   #endif
     if (pid) { /* original process */
         if (pid < 0) return 0;
@@ -1011,7 +1182,7 @@ static int server_graceful_state_bg (server *srv) {
 
     /* (original process is backgrounded -- even if no active connections --
      *  to allow graceful shutdown tasks to be run by server and by modules) */
-    log_error(srv->errh, __FILE__, __LINE__,
+    log_notice(srv->errh, __FILE__, __LINE__,
       "[note] pid %lld continuing to handle %u connection(s) in progress",
       (long long)getpid(), srv->srvconf.max_conns - srv->lim_conns);
 
@@ -1035,6 +1206,10 @@ static void server_graceful_shutdown_maint (server *srv) {
     connection_graceful_shutdown_maint(srv);
 }
 
+#ifndef server_status_stopping
+#define server_status_stopping(srv) do { } while (0)
+#endif
+
 __attribute_cold__
 __attribute_noinline__
 static void server_graceful_state (server *srv) {
@@ -1049,12 +1224,14 @@ static void server_graceful_state (server *srv) {
         server_graceful_shutdown_maint(srv);
     }
 
+    server_status_stopping(srv);/*might be called multiple times; intentional*/
+
     if (2 == srv->sockets_disabled || 3 == srv->sockets_disabled) {
         if (oneshot_fd) graceful_restart = 0;
         return;
     }
 
-    log_error(srv->errh,__FILE__,__LINE__,"[note] graceful shutdown started");
+    log_notice(srv->errh,__FILE__,__LINE__,"[note] graceful shutdown started");
 
     /* no graceful restart if chroot()ed, if oneshot mode, or if idle timeout */
     if (srv->srvconf.changeroot || oneshot_fd || 2 == graceful_shutdown)
@@ -1078,7 +1255,7 @@ __attribute_noinline__
 static void server_sockets_enable (server *srv) {
     server_sockets_set_event(srv, FDEVENT_IN);
     srv->sockets_disabled = 0;
-    log_error(srv->errh, __FILE__, __LINE__, "[note] sockets enabled again");
+    log_notice(srv->errh, __FILE__, __LINE__, "[note] sockets enabled again");
 }
 
 __attribute_cold__
@@ -1086,7 +1263,7 @@ __attribute_noinline__
 static void server_sockets_disable (server *srv) {
     server_sockets_set_event(srv, 0);
     srv->sockets_disabled = 1;
-    log_error(srv->errh, __FILE__, __LINE__,
+    log_notice(srv->errh, __FILE__, __LINE__,
       (0 == srv->lim_conns)
         ? "[note] sockets disabled, connection limit reached"
         : "[note] sockets disabled, out-of-fds");
@@ -1238,6 +1415,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 	int i_am_root = 0;
 #ifdef HAVE_FORK
 	int parent_pipe_fd = -1;
+	const char *conffile = NULL;
 #endif
 
 #ifdef HAVE_GETUID
@@ -1271,6 +1449,9 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			if (config_read(srv, optarg)) {
 				return -1;
 			}
+#ifdef HAVE_FORK
+			conffile = optarg;
+#endif
 			break;
 		case 'm':
 			srv->srvconf.modules_dir = optarg;
@@ -1300,7 +1481,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		}
 	}
 
-      #ifdef __CYGWIN__
+      #if defined(__CYGWIN__) || defined(_WIN32)
 	if (!srv->config_data_base && NULL != getenv("NSSM_SERVICE_NAME")) {
 		char *dir = getenv("NSSM_SERVICE_DIR");
 		if (NULL != dir && 0 != chdir(dir)) {
@@ -1311,6 +1492,9 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		srv->srvconf.modules_dir = "modules";
 		if (config_read(srv, "conf/lighttpd.conf")) return -1;
 	}
+      #ifndef HAVE_FORK
+	srv->srvconf.dont_daemonize = 1;
+      #endif
       #endif
 
 	if (!srv->config_data_base) {
@@ -1359,7 +1543,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		srv->srvconf.pid_file = NULL;
 		if (srv->srvconf.max_worker) {
 			srv->srvconf.max_worker = 0;
-			log_error(srv->errh, __FILE__, __LINE__,
+			log_warn(srv->errh, __FILE__, __LINE__,
 			  "server one-shot command line option disables server.max-worker config file option.");
 		}
 
@@ -1369,6 +1553,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			return -1;
 		}
 
+	  #ifndef _WIN32 /*(skip S_ISFIFO() and hope for the best if _WIN32)*/
 		if (S_ISFIFO(st.st_mode)) {
 			oneshot_fdout = dup(STDOUT_FILENO);
 			if (oneshot_fdout <= STDERR_FILENO) {
@@ -1376,6 +1561,8 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 				return -1;
 			}
 		}
+	  #endif
+	  #ifndef _WIN32 /*(skip S_ISSOCK() and hope for the best if _WIN32)*/
 		else if (!S_ISSOCK(st.st_mode)) {
 			/* require that fd is a socket
 			 * (modules might expect STDIN_FILENO and STDOUT_FILENO opened to /dev/null) */
@@ -1383,9 +1570,13 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			  "lighttpd -1 stdin is not a socket");
 			return -1;
 		}
+	  #endif
 	}
 
 	if (srv->srvconf.bindhost && buffer_is_equal_string(srv->srvconf.bindhost, CONST_STR_LEN("/dev/stdin"))) {
+		/* XXX: to potentially support on _WIN32,
+		 *      (SOCKET)GetStdHandle(STD_INPUT_HANDLE) and
+		 *      WSADuplicateSocket() instead of dup() */
 		if (-1 == srv->stdin_fd)
 			srv->stdin_fd = dup(STDIN_FILENO);
 		if (srv->stdin_fd <= STDERR_FILENO) {
@@ -1396,6 +1587,27 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 	}
 
 	/* close stdin and stdout, as they are not needed */
+  #ifdef _WIN32
+	/* _WIN32 file descriptors are not allocated lowest first.
+	 * Open NUL in binary mode and as (default) inheritable handle
+	 * https://learn.microsoft.com/en-us/cpp/c-runtime-library/reference/dup-dup2?view=msvc-170
+	 *
+	 * A stream is associated with a file descriptor (_fileno(stream)).
+	 * An open file descriptor has an underlying operating system HANDLE.
+	 * However, standard handles are cached at program startup,
+	 * so we try to match them all back up after redirection. */
+	if (   NULL == freopen("nul:", "rb", stdin)
+	    || NULL == freopen("nul:", "wb", stdout)
+	    || (_fileno(stderr) == -2
+		&& NULL == freopen("nul:", "wb", stderr))) {
+		log_perror(srv->errh, __FILE__, __LINE__, "freopen() NUL");
+		return -1;
+	}
+	SetStdHandle(STD_INPUT_HANDLE, (HANDLE)_get_osfhandle(_fileno(stdin)));
+	SetStdHandle(STD_OUTPUT_HANDLE,(HANDLE)_get_osfhandle(_fileno(stdout)));
+	SetStdHandle(STD_ERROR_HANDLE, (HANDLE)_get_osfhandle(_fileno(stderr)));
+	fdevent_setfd_cloexec(STDERR_FILENO);
+  #else
 	{
 		struct stat st;
 		int devnull;
@@ -1427,14 +1639,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		if (devnull != errfd) close(devnull);
 	      #endif
 	}
-
-	http_response_send_1xx_cb_set(NULL, HTTP_VERSION_2);
-	if (!config_feature_bool(srv, "server.h2-discard-backend-1xx", 0))
-		http_response_send_1xx_cb_set(h2_send_1xx, HTTP_VERSION_2);
-
-	http_response_send_1xx_cb_set(NULL, HTTP_VERSION_1_1);
-	if (!config_feature_bool(srv, "server.h1-discard-backend-1xx", 0))
-		http_response_send_1xx_cb_set(connection_send_1xx, HTTP_VERSION_1_1);
+  #endif
 
 	http_range_config_allow_http10(config_feature_bool(srv, "http10.range", 0));
 
@@ -1456,13 +1661,27 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		return -1;
 	}
 
+	http_dispatch[HTTP_VERSION_1_1] = h1_1_dispatch_table; /* copy struct */
+
+	if (config_feature_bool(srv, "server.h2-discard-backend-1xx", 0))
+		http_dispatch[HTTP_VERSION_2].send_1xx = 0;
+
+	if (config_feature_bool(srv, "server.h1-discard-backend-1xx", 0))
+		http_dispatch[HTTP_VERSION_1_1].send_1xx = 0;
+
+	if (config_feature_bool(srv, "server.discard-backend-1xx", 0)) {
+		http_dispatch[HTTP_VERSION_3].send_1xx = 0;
+		http_dispatch[HTTP_VERSION_2].send_1xx = 0;
+		http_dispatch[HTTP_VERSION_1_1].send_1xx = 0;
+	}
+
 	/* mod_indexfile should be listed in server.modules prior to dynamic handlers */
 	uint32_t i = 0;
 	for (const char *pname = NULL; i < srv->plugins.used; ++i) {
 		plugin *p = ((plugin **)srv->plugins.ptr)[i];
 		if (0 == strcmp(p->name, "indexfile")) {
 			if (pname)
-				log_error(srv->errh, __FILE__, __LINE__,
+				log_warn(srv->errh, __FILE__, __LINE__,
 				  "Warning: mod_indexfile should be listed in server.modules prior to mod_%s", pname);
 			break;
 		}
@@ -1546,6 +1765,12 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			rlim.rlim_cur = rlim.rlim_max;
 			setrlimit(RLIMIT_CORE, &rlim);
 		}
+#else
+	  #ifdef _WIN32
+		/*(default upper limit of 4k if server.max-fds not specified)*/
+		if (0 == srv->srvconf.max_fds)
+			srv->srvconf.max_fds = 4096;
+	  #endif
 #endif
 	}
 
@@ -1665,6 +1890,18 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 #ifdef HAVE_FORK
 	/* network is up, let's daemonize ourself */
 	if (0 == srv->srvconf.dont_daemonize && 0 == graceful_restart) {
+		if (conffile && *conffile != '/'
+		    && (conffile[0] != '-' || conffile[1] != '\0')/*(special-case "-")*/
+		   #if defined(_WIN32)
+                    && *conffile != '\\'
+                    && conffile[0] && conffile[1] != ':'
+		   #endif
+		    /*(might perform similar checks on srv->srvconf.modules_dir)*/
+		    ) {
+			log_warn(srv->errh, __FILE__, __LINE__,
+			  "(warning) daemonizing without absolute path command line args"
+			  " (graceful restart may fail)");
+		}
 		parent_pipe_fd = daemonize();
 	}
 #endif
@@ -1673,8 +1910,10 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 
 	server_main_setup_signals();
 
+  #ifdef HAVE_GETUID
 	srv->gid = getgid();
 	srv->uid = getuid();
+  #endif
 	srv->pid = getpid();
 
 	/* write pid file */
@@ -1701,7 +1940,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			return -1;
 		}
 		if (!oneshot_fd)
-			log_error(srv->errh, __FILE__, __LINE__, "server started (" PACKAGE_DESC ")");
+			log_notice(srv->errh, __FILE__, __LINE__, "server started (" PACKAGE_DESC ")");
 	}
 
 	if (HANDLER_GO_ON != plugins_call_set_defaults(srv)) {
@@ -1731,7 +1970,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 
 	if (idle_limit && srv->srvconf.max_worker) {
 		srv->srvconf.max_worker = 0;
-		log_error(srv->errh, __FILE__, __LINE__,
+		log_warn(srv->errh, __FILE__, __LINE__,
 		  "server idle time limit command line option disables server.max-worker config file option.");
 	}
 
@@ -1758,7 +1997,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 	/* set max-conns */
 	if (srv->srvconf.max_conns > srv->max_fds/2) {
 		/* we can't have more connections than max-fds/2 */
-		log_error(srv->errh, __FILE__, __LINE__,
+		log_warn(srv->errh, __FILE__, __LINE__,
 		  "can't have more connections than fds/2: %hu %d",
 		  srv->srvconf.max_conns, srv->max_fds);
 		srv->lim_conns = srv->srvconf.max_conns = srv->max_fds/2;
@@ -1788,22 +2027,10 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		return -1;
 	}
 
-#ifdef USE_ALARM
-	{
-		/* setup periodic timer (1 second) */
-		struct itimerval interval;
-		interval.it_interval.tv_sec = 1;
-		interval.it_interval.tv_usec = 0;
-		interval.it_value.tv_sec = 1;
-		interval.it_value.tv_usec = 0;
-		if (setitimer(ITIMER_REAL, &interval, NULL)) {
-			log_perror(srv->errh, __FILE__, __LINE__, "setitimer()");
-			return -1;
-		}
-	}
-#endif
-
 	/* get the current number of FDs */
+  #ifdef _WIN32
+	srv->cur_fds = 3; /*(estimate on _WIN32)*/
+  #else
 	{
 		int fd = fdevent_open_devnull();
 		if (fd >= 0) {
@@ -1811,6 +2038,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			close(fd);
 		}
 	}
+  #endif
 
 	if (0 != server_sockets_set_nb_cloexec(srv)) {
 		return -1;
@@ -1845,12 +2073,12 @@ static void server_handle_sighup (server * const srv) {
 			plugins_call_handle_sighup(srv);
 			fdlog_files_cycle(srv->errh); /* reopen log files, not pipes */
 #ifdef HAVE_SIGACTION
-				log_error(srv->errh, __FILE__, __LINE__,
+				log_info(srv->errh, __FILE__, __LINE__,
 				  "logfiles cycled UID = %d PID = %d",
 				  (int)last_sighup_info.si_uid,
 				  (int)last_sighup_info.si_pid);
 #else
-				log_error(srv->errh, __FILE__, __LINE__,
+				log_info(srv->errh, __FILE__, __LINE__,
 				  "logfiles cycled");
 #endif
 }
@@ -1864,8 +2092,8 @@ static void server_handle_sigalrm (server * const srv, unix_time64_t mono_ts, un
 				log_epoch_secs = server_epoch_secs(srv, 0);
 
 				/* check idle time limit, if enabled */
-				if (idle_limit && idle_limit < mono_ts - last_active_ts && !graceful_shutdown) {
-					log_error(srv->errh, __FILE__, __LINE__,
+				if (idle_limit && (unix_time64_t)idle_limit < mono_ts - last_active_ts && !graceful_shutdown) {
+					log_notice(srv->errh, __FILE__, __LINE__,
 					  "[note] idle timeout %ds exceeded, "
 					  "initiating graceful shutdown", (int)idle_limit);
 					graceful_shutdown = 2; /* value 2 indicates idle timeout */
@@ -1953,8 +2181,8 @@ static void server_main_loop (server * const srv) {
 			server_handle_sighup(srv);
 		}
 
-		/*(USE_ALARM not used; fdevent_poll() is effective periodic timer)*/
-	      #ifdef USE_ALARM
+		/*(SIGALRM not used here; fdevent_poll() is effective periodic timer)*/
+	      #if 0
 		if (handle_sig_alarm) {
 			handle_sig_alarm = 0;
 	      #endif
@@ -1962,7 +2190,7 @@ static void server_main_loop (server * const srv) {
 			if (mono_ts != log_monotonic_secs) {
 				server_handle_sigalrm(srv, mono_ts, last_active_ts);
 			}
-	      #ifdef USE_ALARM
+	      #if 0
 		}
 	      #endif
 
@@ -1985,7 +2213,10 @@ static void server_main_loop (server * const srv) {
 			server_load_check(srv);
 		}
 
-		static connection * const sentinel =
+	  #ifndef _MSC_VER
+		static
+	  #endif
+		connection * const sentinel =
 		  (connection *)(uintptr_t)&log_con_jqueue;
 		connection * const joblist = log_con_jqueue;
 		log_con_jqueue = sentinel;
@@ -2040,14 +2271,43 @@ static int main_init_once (void) {
   #endif
 
     /* for nice %b handling in strftime() */
+  #ifdef _WIN32
+    setlocale(LC_ALL, "C.UTF-8");
+   #ifdef __MINGW32__
+    _setmbcp(_MB_CP_LOCALE);
+   #else
+    _setmbcp(_MB_CP_UTF8);
+   #endif
+  #else
     setlocale(LC_TIME, "C");
+  #endif
     tzset();
 
+  #ifdef __MINGW32__
+    /* MSYS2 translates SHELL path even if MSYS_NO_PATHCONV=1
+     * lighttpd uses "/bin/sh" for consistency and substitutes "cmd.exe" later
+     * (__MINGW32__ is also set when mingw cross-compiler used under Cygwin) */
+    if (getenv("MSYSTEM")) { /* MSYS2 */
+        const char *shell = getenv("SHELL");
+        size_t len = shell ? strlen(shell) : 0;
+        if (len >= 11 && 0 == strcmp(shell+len-11, "\\bin\\sh.exe"))
+            setenv("SHELL", "/bin/sh", 1);
+    }
+  #endif
+
     return 1;
 }
 
+#ifndef server_status_running
+#define server_status_running(srv) do { } while (0)
+#endif
+
+#ifndef main
+#define server_main main
+#endif
+
 __attribute_cold__
-int main (int argc, char ** argv) {
+int server_main (int argc, char ** argv) {
     if (!main_init_once()) return -1;
 
     int rc;
@@ -2062,6 +2322,7 @@ int main (int argc, char ** argv) {
 
         rc = server_main_setup(srv, argc, argv);
         if (rc > 0) {
+            server_status_running(srv);
 
             server_main_loop(srv);
 
@@ -2071,16 +2332,16 @@ int main (int argc, char ** argv) {
 
             if (NULL == srv->conns) rc = 0;
             if (2 == graceful_shutdown) { /* value 2 indicates idle timeout */
-                log_error(srv->errh, __FILE__, __LINE__,
+                log_notice(srv->errh, __FILE__, __LINE__,
                   "server stopped after idle timeout");
             } else if (!oneshot_fd) {
               #ifdef HAVE_SIGACTION
-                log_error(srv->errh, __FILE__, __LINE__,
+                log_notice(srv->errh, __FILE__, __LINE__,
                   "server stopped by UID = %d PID = %d",
                   (int)last_sigterm_info.si_uid,
                   (int)last_sigterm_info.si_pid);
               #else
-                log_error(srv->errh, __FILE__, __LINE__,
+                log_notice(srv->errh, __FILE__, __LINE__,
                   "server stopped");
               #endif
             }
@@ -2090,6 +2351,9 @@ int main (int argc, char ** argv) {
         chunkqueue_internal_pipes(0);
         remove_pid_file(srv);
         config_log_error_close(srv);
+      #ifdef _WIN32
+        fdevent_win32_cleanup();
+      #endif
         if (graceful_restart)
             server_sockets_save(srv);
         else
